@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -29,6 +29,8 @@
 #include <libdaemon/dfork.h>
 #include <libdaemon/dpid.h>
 #include <libdaemon/dlog.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #endif
 
 #include <locale.h>
@@ -100,29 +102,37 @@ infinoted_main_run(InfinotedStartup* startup,
     }
     else
     {
-      if(!infinoted_util_set_daemon_pid_file_proc(error))
-      {
-        infinoted_run_free(run);
-        return FALSE;
-      }
-
+      infinoted_util_daemon_set_global_pid_file_proc();
       if(daemon_pid_file_create() != 0)
       {
-        saved_errno = errno;
-        infinoted_util_set_errno_error(
-          error,
-          saved_errno,
-          _("Failed to create PID file")
-        );
-        if(saved_errno < 0) --saved_errno;
-        daemon_retval_send(saved_errno);
+        infinoted_util_daemon_set_local_pid_file_proc();
+        if(daemon_pid_file_create() != 0)
+        {
+          if(daemon_pid_file_create() != 0)
+          {
+            saved_errno = errno;
+            infinoted_util_set_errno_error(
+              error,
+              saved_errno,
+              _("Failed to create PID file")
+            );
+            if(saved_errno < 0) --saved_errno;
+            daemon_retval_send(saved_errno);
 
-        infinoted_run_free(run);
-        return FALSE;
+            infinoted_run_free(run);
+            return FALSE;
+          }
+        }
       }
 
       daemon_retval_send(0);
     }
+
+    /* libdaemon < 0.14 sets our umask to 0777, preventing the file storage
+     * backend from working correctly. 0.14 uses 0077, but the documentation
+     * still says 0777, and, anyway, 0.14 does not seem widespread enough
+     * right now. */
+    umask(0077);
   }
 #endif
 
