@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -39,6 +39,11 @@ struct _InfinotedOptions {
   gchar* root_directory;
   guint autosave_interval;
   gchar* password;
+#ifdef LIBINFINITY_HAVE_PAM
+  gchar* pam_service;
+  gchar** pam_allowed_users;
+  gchar** pam_allowed_groups;
+#endif /* LIBINFINITY_HAVE_PAM */
 
   gchar* sync_directory;
   guint sync_interval;
@@ -55,7 +60,8 @@ typedef enum _InfinotedOptionsError {
   INFINOTED_OPTIONS_ERROR_INVALID_CREATE_OPTIONS,
   INFINOTED_OPTIONS_ERROR_EMPTY_KEY_FILE,
   INFINOTED_OPTIONS_ERROR_EMPTY_CERTIFICATE_FILE,
-  INFINOTED_OPTIONS_ERROR_INVALID_SYNC_COMBINATION
+  INFINOTED_OPTIONS_ERROR_INVALID_SYNC_COMBINATION,
+  INFINOTED_OPTIONS_ERROR_INVALID_AUTHENTICATION_SETTINGS
 } InfinotedOptionsError;
 
 InfinotedOptions*
