@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,8 +32,6 @@
 #include <libinfinity/common/inf-standalone-io.h>
 #include <libinfinity/common/inf-discovery-avahi.h>
 
-#include <gsasl.h>
-
 #include <glib.h>
 
 G_BEGIN_DECLS
@@ -51,7 +49,6 @@ struct _InfinotedRun {
   InfdXmppServer* xmpp4;
   InfdXmppServer* xmpp6;
   gnutls_dh_params_t dh_params;
-  Gsasl* gsasl;
 
   InfinotedRecord* record;
 
