@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,14 +21,18 @@
 #include <infinoted/infinoted-util.h>
 #include <infinoted/infinoted-creds.h>
 
+#include <infinoted/infinoted-pam.h>
+
 #include <libinfinity/common/inf-cert-util.h>
 #include <libinfinity/common/inf-init.h>
 #include <libinfinity/common/inf-error.h>
 #include <libinfinity/inf-i18n.h>
+#include <libinfinity/inf-config.h>
 
 #include <gnutls/x509.h>
 
 #include <string.h>
+#include <stdlib.h>
 
 static void
 infinoted_startup_free_certificate_array(gnutls_x509_crt_t* certificates,
@@ -225,25 +229,120 @@ infinoted_startup_load_options(InfinotedStartup* startup,
   return TRUE;
 }
 
-static int
-infinoted_startup_gsasl_callback(Gsasl* gsasl,
-                                 Gsasl_session* session,
-                                 Gsasl_property prop)
+static void
+infinoted_startup_sasl_callback_set_error(InfXmppConnection* connection,
+                                          InfAuthenticationDetailError code,
+                                          const GError* error)
+{
+  GError* own_error;
+  own_error = NULL;
+
+  if(!error)
+  {
+    own_error = g_error_new_literal(
+      inf_authentication_detail_error_quark(),
+      code,
+      inf_authentication_detail_strerror(code)
+    );
+
+    inf_xmpp_connection_set_sasl_error(connection, own_error);
+    g_error_free(own_error);
+  }
+  else
+  {
+    inf_xmpp_connection_set_sasl_error(connection, error);
+  }
+}
+
+static void
+infinoted_startup_sasl_callback(InfSaslContextSession* session,
+                                Gsasl_property prop,
+                                gpointer session_data,
+                                gpointer user_data)
 {
-  const char* prop_ptr;
   InfinotedStartup* startup;
+  const char* password;
+  InfXmppConnection* xmpp;
+
+#ifdef LIBINFINITY_HAVE_PAM
+  const char* username;
+  const gchar* pam_service;
+  GError* error;
+#endif
+
+  xmpp = INF_XMPP_CONNECTION(session_data);
+
   switch(prop)
   {
   case GSASL_VALIDATE_SIMPLE:
-    startup = gsasl_callback_hook_get(gsasl);
-    g_assert(startup->options->password != NULL);
-    prop_ptr = gsasl_property_fast(session, GSASL_PASSWORD);
-    if(strcmp(startup->options->password, prop_ptr) == 0)
-      return GSASL_OK;
+    startup = (InfinotedStartup*)user_data;
+    password = inf_sasl_context_session_get_property(session, GSASL_PASSWORD);
+#ifdef LIBINFINITY_HAVE_PAM
+    username = inf_sasl_context_session_get_property(session, GSASL_AUTHID);
+    pam_service = startup->options->pam_service;
+    if(pam_service != NULL)
+    {
+      error = NULL;
+      if(!infinoted_pam_authenticate(pam_service, username, password))
+      {
+        infinoted_startup_sasl_callback_set_error(
+          xmpp,
+          INF_AUTHENTICATION_DETAIL_ERROR_AUTHENTICATION_FAILED,
+          NULL
+        );
+
+        inf_sasl_context_session_continue(
+          session,
+          GSASL_AUTHENTICATION_ERROR
+        );
+      }
+      else if(!infinoted_pam_user_is_allowed(startup->options,
+                                             username,
+                                             &error))
+      {
+        infinoted_startup_sasl_callback_set_error(
+          xmpp,
+          INF_AUTHENTICATION_DETAIL_ERROR_USER_NOT_AUTHORIZED,
+          error
+        );
+
+        inf_sasl_context_session_continue(
+          session,
+          GSASL_AUTHENTICATION_ERROR
+        );
+      }
+      else
+      {
+        inf_sasl_context_session_continue(session, GSASL_OK);
+      }
+    }
     else
-      return GSASL_AUTHENTICATION_ERROR;
+#endif /* LIBINFINITY_HAVE_PAM */
+    {
+      g_assert(startup->options->password != NULL);
+      if(strcmp(startup->options->password, password) == 0)
+      {
+        inf_sasl_context_session_continue(session, GSASL_OK);
+      }
+      else
+      {
+        infinoted_startup_sasl_callback_set_error(
+          xmpp,
+          INF_AUTHENTICATION_DETAIL_ERROR_AUTHENTICATION_FAILED,
+          NULL
+        );
+
+        inf_sasl_context_session_continue(
+          session,
+          GSASL_AUTHENTICATION_ERROR
+        );
+      }
+    }
+
+    break;
   default:
-    return GSASL_AUTHENTICATION_ERROR;
+    inf_sasl_context_session_continue(session, GSASL_AUTHENTICATION_ERROR);
+    break;
   }
 }
 
@@ -253,7 +352,7 @@ infinoted_startup_load(InfinotedStartup* startup,
                        char*** argv,
                        GError** error)
 {
-  int gsasl_status;
+  gboolean requires_password;
 
   if(infinoted_startup_load_options(startup, argc, argv, error) == FALSE)
     return FALSE;
@@ -261,16 +360,22 @@ infinoted_startup_load(InfinotedStartup* startup,
   if(infinoted_startup_load_credentials(startup, error) == FALSE)
     return FALSE;
 
-  if(startup->options->password)
+  requires_password = startup->options->password != NULL;
+#ifdef LIBINFINITY_HAVE_PAM
+  requires_password =
+    requires_password || startup->options->pam_service != NULL;
+#endif /* LIBINFINITY_HAVE_PAM */
+
+  if(requires_password)
   {
-    gsasl_status = gsasl_init(&startup->gsasl);
-    if (gsasl_status != GSASL_OK)
-    {
-      inf_gsasl_set_error(error, gsasl_status);
-      return FALSE;
-    }
-    gsasl_callback_set(startup->gsasl, infinoted_startup_gsasl_callback);
-    gsasl_callback_hook_set(startup->gsasl, startup);
+    startup->sasl_context = inf_sasl_context_new(error);
+    if(!startup->sasl_context) return FALSE;
+
+    inf_sasl_context_set_callback(
+      startup->sasl_context,
+      infinoted_startup_sasl_callback,
+      startup
+    );
   }
 
   return TRUE;
@@ -302,7 +407,7 @@ infinoted_startup_new(int* argc,
   startup->certificates = NULL;
   startup->n_certificates = 0;
   startup->credentials = NULL;
-  startup->gsasl = NULL;
+  startup->sasl_context = NULL;
 
   if(infinoted_startup_load(startup, argc, argv, error) == FALSE)
   {
@@ -339,8 +444,8 @@ infinoted_startup_free(InfinotedStartup* startup)
   if(startup->options != NULL)
     infinoted_options_free(startup->options);
 
-  if(startup->gsasl != NULL)
-    gsasl_done(startup->gsasl);
+  if(startup->sasl_context != NULL)
+    inf_sasl_context_unref(startup->sasl_context);
 
   g_slice_free(InfinotedStartup, startup);
   inf_deinit();
