@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,8 +23,6 @@
 #include <infinoted/infinoted-options.h>
 #include <libinfinity/common/inf-certificate-credentials.h>
 
-#include <gsasl.h>
-
 #include <glib.h>
 
 G_BEGIN_DECLS
@@ -36,7 +34,7 @@ struct _InfinotedStartup {
   gnutls_x509_crt_t* certificates;
   guint n_certificates;
   InfCertificateCredentials* credentials;
-  Gsasl* gsasl;
+  InfSaslContext* sasl_context;
 };
 
 InfinotedStartup*
