@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,7 +23,7 @@
 #include <libinfinity/client/infc-browser.h>
 #include <libinfinity/common/inf-discovery.h>
 
-#include <gtk/gtktreemodel.h>
+#include <gtk/gtk.h>
 
 #include <glib-object.h>
 
