@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,7 +24,7 @@
 #include <libinfinity/inf-i18n.h>
 #include <libinfinity/inf-signals.h>
 
-#include <gtk/gtktreemodel.h>
+#include <gtk/gtk.h>
 
 /* The three pointers in GtkTreeIter are used as follows:
  *
