@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -91,7 +91,8 @@ inf_gtk_certificate_dialog_renew_info(InfGtkCertificateDialog* dialog)
     caption = gtk_label_new(NULL);
     gtk_misc_set_alignment(GTK_MISC(caption), 0.0, 0.0);
     gtk_label_set_line_wrap(GTK_LABEL(caption), TRUE);
-    size = pango_font_description_get_size(caption->style->font_desc);
+    size = pango_font_description_get_size(
+      gtk_widget_get_style(caption)->font_desc);
     font_desc = pango_font_description_new();
     pango_font_description_set_weight(font_desc, PANGO_WEIGHT_BOLD);
     pango_font_description_set_size(font_desc, size * PANGO_SCALE_LARGE);
@@ -398,6 +399,7 @@ inf_gtk_certificate_dialog_init(GTypeInstance* instance,
   GtkWidget* image;
   GtkWidget* hbox;
   GtkWidget* scroll;
+  GtkWidget* vbox;
   GtkTreeViewColumn* column;
   GtkCellRenderer* renderer;
   GtkTreeSelection* selection;
@@ -575,15 +577,15 @@ inf_gtk_certificate_dialog_init(GTypeInstance* instance,
 
   gtk_widget_show(priv->main_vbox);
 
-  gtk_box_pack_start(
-    GTK_BOX(GTK_DIALOG(dialog)->vbox),
-    priv->main_vbox,
-    TRUE,
-    TRUE,
-    0
-  );
+#if GTK_CHECK_VERSION(2,14,0)
+  vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
+#else
+  vbox = GTK_DIALOG(dialog)->vbox;
+#endif
 
-  gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox), 12);
+  gtk_box_pack_start(GTK_BOX(vbox), priv->main_vbox, TRUE, TRUE, 0);
+
+  gtk_box_set_spacing(GTK_BOX(vbox), 12);
 
   gtk_container_set_border_width(GTK_CONTAINER(dialog), 12);
   gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
@@ -857,9 +859,11 @@ inf_gtk_certificate_dialog_new(GtkWindow* parent,
   if(dialog_flags & GTK_DIALOG_DESTROY_WITH_PARENT)
     gtk_window_set_destroy_with_parent(GTK_WINDOW(object), TRUE);
 
+#if !GTK_CHECK_VERSION(2,90,7)
   if(dialog_flags & GTK_DIALOG_NO_SEPARATOR)
     gtk_dialog_set_has_separator(GTK_DIALOG(object), FALSE);
- 
+#endif
+
   gtk_window_set_transient_for(GTK_WINDOW(object), parent);
   return INF_GTK_CERTIFICATE_DIALOG(object);
 }
