@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -70,7 +70,7 @@ struct _InfAdoptedSessionPrivate {
   GSList* local_users; /* having zero or one item in 99.9% of all cases */
 
   /* Timeout for sending noop with our current vector time */
-  gpointer noop_timeout;
+  InfIoTimeout* noop_timeout;
   /* User to send the time for */
   InfAdoptedSessionLocalUser* next_noop_user;
 };
@@ -1382,9 +1382,10 @@ inf_adopted_session_get_io(InfAdoptedSession* session)
  * inf_adopted_session_get_algorithm:
  * @session: A #InfAdoptedSession.
  *
- * Returns the #InfAlgorithm object of @session. Returns %NULL if @session
- * has status %INF_SESSION_PRESYNC or %INF_SESSION_SYNCHRONIZING because there
- * the algorithm object is not yet created before successful synchronization.
+ * Returns the #InfAdoptedAlgorithm object of @session. Returns %NULL if
+ * @session has status %INF_SESSION_PRESYNC or %INF_SESSION_SYNCHRONIZING
+ * because there the algorithm object is not yet created before successful
+ * synchronization.
  *
  * Return Value: A #InfAdoptedAlgorithm, or %NULL.
  **/
@@ -1515,12 +1516,12 @@ inf_adopted_session_read_request_info(InfAdoptedSession* session,
     *user = inf_adopted_session_user_from_request_xml(session, xml, error);
     if(*user == NULL) return FALSE;
   }
-  
+
   if(time != NULL)
   {
     attr = inf_xml_util_get_attribute_required(xml, "time", error);
     if(attr == NULL) return FALSE;
-    
+
     if(diff_vec == NULL)
     {
       *time = inf_adopted_state_vector_from_string((const gchar*)attr, error);
