@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -29,18 +29,20 @@ G_BEGIN_DECLS
 
 /* TODO: GTypeModule stuff? */
 
+typedef InfSession*(*InfcNotePluginSessionNew)(InfIo*,
+                                               InfCommunicationManager*,
+                                               InfSessionStatus,
+                                               InfCommunicationJoinedGroup*,
+                                               InfXmlConnection*,
+                                               gpointer);
+
 typedef struct _InfcNotePlugin InfcNotePlugin;
 struct _InfcNotePlugin {
   gpointer user_data;
 
   const gchar* note_type;
 
-  InfSession*(*session_new)(InfIo* io,
-                            InfCommunicationManager* manager,
-                            InfSessionStatus status,
-                            InfCommunicationJoinedGroup* sync_group,
-                            InfXmlConnection* sync_connection,
-                            gpointer user_data);
+  InfcNotePluginSessionNew session_new;
 };
 
 G_END_DECLS
