@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -197,11 +197,14 @@ inf_chat_session_message_from_xml(InfChatSession* session,
   xmlChar* type;
   gboolean result;
   InfChatBufferMessageType message_type;
+  InfChatBufferMessageFlags message_flags;
   long message_time;
   guint user_id;
   InfUserTable* user_table;
   InfUser* user;
 
+  message_flags = 0;
+
   type = inf_xml_util_get_attribute(xml, "type");
   if(type == NULL)
   {
@@ -229,6 +232,7 @@ inf_chat_session_message_from_xml(InfChatSession* session,
     );
 
     if(result == FALSE) return FALSE;
+    message_flags = INF_CHAT_BUFFER_MESSAGE_BACKLOG;
   }
   else
   {
@@ -285,6 +289,7 @@ inf_chat_session_message_from_xml(InfChatSession* session,
   message->type = message_type;
   message->user = user;
   message->time = message_time;
+  message->flags = message_flags;
 
   return TRUE;
 }
@@ -489,6 +494,7 @@ inf_chat_session_user_join(InfChatSession* session,
   message.text = NULL;
   message.length = 0;
   message.time = time(NULL);
+  message.flags = 0;
 
   g_signal_emit(session, chat_session_signals[RECEIVE_MESSAGE], 0, &message);
 }
@@ -504,6 +510,7 @@ inf_chat_session_user_part(InfChatSession* session,
   message.text = NULL;
   message.length = 0;
   message.time = time(NULL);
+  message.flags = 0;
 
   g_signal_emit(session, chat_session_signals[RECEIVE_MESSAGE], 0, &message);
 }
@@ -804,9 +811,9 @@ inf_chat_session_to_xml_sync(InfSession* session,
   g_assert(parent_class->to_xml_sync != NULL);
   parent_class->to_xml_sync(session, parent);
 
-  for(i = inf_chat_buffer_get_n_messages(buffer); i > 0; --i)
+  for(i = 0; i < inf_chat_buffer_get_n_messages(buffer); ++i)
   {
-    message = inf_chat_buffer_get_message(buffer, i - 1);
+    message = inf_chat_buffer_get_message(buffer, i);
 
     child = inf_chat_session_message_to_xml(
       INF_CHAT_SESSION(session),
@@ -945,7 +952,8 @@ inf_chat_session_receive_message_handler(InfChatSession* session,
       message->user,
       message->text,
       message->length,
-      message->time
+      message->time,
+      message->flags
     );
     break;
   case INF_CHAT_BUFFER_MESSAGE_EMOTE:
@@ -954,28 +962,31 @@ inf_chat_session_receive_message_handler(InfChatSession* session,
       message->user,
       message->text,
       message->length,
-      message->time
+      message->time,
+      message->flags
     );
     break;
   case INF_CHAT_BUFFER_MESSAGE_USERJOIN:
     inf_chat_buffer_add_userjoin_message(
       buffer,
       message->user,
-      message->time
+      message->time,
+      message->flags
     );
     break;
   case INF_CHAT_BUFFER_MESSAGE_USERPART:
     inf_chat_buffer_add_userpart_message(
       buffer,
       message->user,
-      message->time
+      message->time,
+      message->flags
     );
     break;
   default:
     g_assert_not_reached();
     break;
   }
-  
+
   inf_signal_handlers_unblock_by_func(
     buffer,
     G_CALLBACK(inf_chat_session_add_message_cb),
@@ -1081,7 +1092,7 @@ inf_chat_session_class_init(gpointer g_class,
   /**
    * InfChatSession::send-message:
    * @session: The #InfChatSession that is sending a message.
-   * @message: The #InfChatSessionMessage that is sent.
+   * @message: The #InfChatBufferMessage that is sent.
    *
    * This signal is emitted whenever a message is sent. Messages can be sent
    * by calling inf_chat_buffer_add_message() or
