@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -54,6 +54,7 @@
 
 struct AvahiWatch {
   InfDiscoveryAvahi* avahi;
+  InfIoWatch* watch;
 
   InfNativeSocket socket;
   AvahiWatchEvent occured_events;
@@ -64,7 +65,7 @@ struct AvahiWatch {
 
 struct AvahiTimeout {
   InfDiscoveryAvahi* avahi;
-  gpointer timeout_handle;
+  InfIoTimeout* timeout;
   AvahiTimeoutCallback callback;
   void* userdata;
 };
@@ -114,7 +115,7 @@ struct _InfDiscoveryAvahiPrivate {
   InfXmppManager* xmpp_manager;
   InfXmppConnectionSecurityPolicy security_policy;
   InfCertificateCredentials* creds;
-  Gsasl* sasl_context;
+  InfSaslContext* sasl_context;
   gchar* sasl_mechanisms;
 
   AvahiClient* client;
@@ -907,7 +908,7 @@ inf_discovery_avahi_timeout_cb(gpointer user_data)
   AvahiTimeout* timeout;
   timeout = (AvahiTimeout*)user_data;
 
-  timeout->timeout_handle = NULL;
+  timeout->timeout = NULL;
   timeout->callback(timeout, timeout->userdata);
 }
 
@@ -932,7 +933,7 @@ inf_discovery_avahi_watch_new(const AvahiPoll* api,
   watch->callback = callback;
   watch->userdata = userdata;
 
-  inf_io_watch(
+  watch->watch = inf_io_add_watch(
     priv->io,
     &watch->socket,
     inf_discovery_avahi_to_io_event(event),
@@ -951,13 +952,10 @@ inf_discovery_avahi_watch_update(AvahiWatch* watch,
   InfDiscoveryAvahiPrivate* priv;
   priv = INF_DISCOVERY_AVAHI_PRIVATE(watch->avahi);
 
-  inf_io_watch(
+  inf_io_update_watch(
     priv->io,
-    &watch->socket,
-    inf_discovery_avahi_to_io_event(event),
-    inf_discovery_avahi_watch_cb,
-    watch,
-    NULL
+    watch->watch,
+    inf_discovery_avahi_to_io_event(event)
   );
 }
 
@@ -973,15 +971,7 @@ inf_discovery_avahi_watch_free(AvahiWatch* watch)
   InfDiscoveryAvahiPrivate* priv;
   priv = INF_DISCOVERY_AVAHI_PRIVATE(watch->avahi);
 
-  inf_io_watch(
-    priv->io,
-    &watch->socket,
-    0,
-    NULL,
-    NULL,
-    NULL
-  );
-
+  inf_io_remove_watch(priv->io, watch->watch);
   g_slice_free(AvahiWatch, watch);
 }
 
@@ -1010,7 +1000,7 @@ inf_discovery_avahi_timeout_new(const AvahiPoll* api,
     usec = avahi_age(tv);
     if(usec > 0) usec = 0;
 
-    timeout->timeout_handle = inf_io_add_timeout(
+    timeout->timeout = inf_io_add_timeout(
       priv->io,
       ((-usec) + 500) / 1000,
       inf_discovery_avahi_timeout_cb,
@@ -1020,7 +1010,7 @@ inf_discovery_avahi_timeout_new(const AvahiPoll* api,
   }
   else
   {
-    timeout->timeout_handle = NULL;
+    timeout->timeout = NULL;
   }
 
   return timeout;
@@ -1034,8 +1024,8 @@ inf_discovery_avahi_timeout_update(AvahiTimeout* timeout,
   AvahiUsec usec;
   priv = INF_DISCOVERY_AVAHI_PRIVATE(timeout->avahi);
 
-  if(timeout->timeout_handle != NULL)
-    inf_io_remove_timeout(priv->io, timeout->timeout_handle);
+  if(timeout->timeout != NULL)
+    inf_io_remove_timeout(priv->io, timeout->timeout);
 
   if(tv != NULL)
   {
@@ -1043,7 +1033,7 @@ inf_discovery_avahi_timeout_update(AvahiTimeout* timeout,
     usec = avahi_age(tv);
     if(usec > 0) usec = 0;
 
-    timeout->timeout_handle = inf_io_add_timeout(
+    timeout->timeout = inf_io_add_timeout(
       priv->io,
       ((-usec) + 500) / 1000,
       inf_discovery_avahi_timeout_cb,
@@ -1053,7 +1043,7 @@ inf_discovery_avahi_timeout_update(AvahiTimeout* timeout,
   }
   else
   {
-    timeout->timeout_handle = NULL;
+    timeout->timeout = NULL;
   }
 }
 
@@ -1063,8 +1053,8 @@ inf_discovery_avahi_timeout_free(AvahiTimeout* timeout)
   InfDiscoveryAvahiPrivate* priv;
   priv = INF_DISCOVERY_AVAHI_PRIVATE(timeout->avahi);
 
-  if(timeout->timeout_handle != NULL)
-    inf_io_remove_timeout(priv->io, timeout->timeout_handle);
+  if(timeout->timeout != NULL)
+    inf_io_remove_timeout(priv->io, timeout->timeout);
 
   g_slice_free(AvahiTimeout, timeout);
 }
@@ -1168,6 +1158,12 @@ inf_discovery_avahi_dispose(GObject* object)
     inf_certificate_credentials_unref(priv->creds);
     priv->creds = NULL;
   }
+  
+  if(priv->sasl_context != NULL)
+  {
+    inf_sasl_context_unref(priv->sasl_context);
+    priv->sasl_context = NULL;
+  }
 
   if(priv->io != NULL)
   {
@@ -1219,7 +1215,7 @@ inf_discovery_avahi_set_property(GObject* object,
     priv->creds = (InfCertificateCredentials*)g_value_dup_boxed(value);
     break;
   case PROP_SASL_CONTEXT:
-    priv->sasl_context = (Gsasl*)g_value_get_pointer(value);
+    priv->sasl_context = (InfSaslContext*)g_value_dup_boxed(value);
     break;
   case PROP_SASL_MECHANISMS:
     g_free(priv->sasl_mechanisms);
@@ -1258,7 +1254,7 @@ inf_discovery_avahi_get_property(GObject* object,
     g_value_set_boxed(value, priv->creds);
     break;
   case PROP_SASL_CONTEXT:
-    g_value_set_pointer(value, (gpointer)priv->sasl_context);
+    g_value_set_boxed(value, priv->sasl_context);
     break;
   case PROP_SASL_MECHANISMS:
     g_value_set_string(value, priv->sasl_mechanisms);
@@ -1551,10 +1547,11 @@ inf_discovery_avahi_class_init(gpointer g_class,
   g_object_class_install_property(
     object_class,
     PROP_SASL_CONTEXT,
-    g_param_spec_pointer(
+    g_param_spec_boxed(
       "sasl-context",
       "SASL context",
-      "The Gsasl context used for authentication",
+      "The SASL context used for authentication",
+      INF_TYPE_SASL_CONTEXT,
       G_PARAM_READWRITE | G_PARAM_CONSTRUCT
     )
   );
@@ -1676,7 +1673,7 @@ inf_discovery_avahi_get_type(void)
  * @io: A #InfIo object used for watching sockets and timeouts.
  * @manager: A #InfXmppManager.
  * @creds: The certificate credentials used for GnuTLS encryption.
- * @sasl_context: A Gsasl context used for authentication.
+ * @sasl_context: A SASL context used for authentication.
  * @sasl_mechanisms: A whitespace-separated list of accepted SASL mechanisms,
  * or %NULL.
  *
@@ -1703,7 +1700,7 @@ InfDiscoveryAvahi*
 inf_discovery_avahi_new(InfIo* io,
                         InfXmppManager* manager,
                         InfCertificateCredentials* creds,
-                        Gsasl* sasl_context,
+                        InfSaslContext* sasl_context,
                         const gchar* sasl_mechanisms)
 {
   GObject* object;
