@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,6 +20,7 @@
 #include <libinfinity/common/inf-init.h>
 #include <libinfinity/inf-i18n.h>
 
+#include <unistd.h> /* Get ssize_t on MSVC, required by gnutls.h */
 #include <gnutls/gnutls.h>
 #include <glib-object.h>
 
@@ -50,6 +51,9 @@ inf_init(GError** error)
   gchar* error_message;
 #endif
 
+  if(!g_thread_supported())
+    g_thread_init(NULL);
+
   if(inf_init_counter == 0)
   {
 #ifdef G_OS_WIN32
@@ -81,7 +85,7 @@ inf_init(GError** error)
   else
     INF_XMPP_CONNECTION_PRINT_TRAFFIC = FALSE;
 
-  ++ inf_init_counter;
+  ++inf_init_counter;
   return TRUE;
 }
 
