@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -33,6 +33,9 @@ inf_protocol_parse_version(const gchar* version,
                            guint* minor,
                            GError** error);
 
+guint
+inf_protocol_get_default_port(void);
+
 G_END_DECLS
 
 #endif /* __INF_PROTOCOL_H__ */
