@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -43,15 +43,15 @@
  * To be notified about changes other users make to a session, you need to
  * subscribe to the session (on client side), or wait for incoming
  * subscriptions (on server side). This is normally done by
- * infc_browser_subscribe_session(). The first action that is performed upon
- * subscription is a synchronization as described above. When the
+ * infc_browser_iter_subscribe_session(). The first action that is performed
+ * upon subscription is a synchronization as described above. When the
  * synchronization is complete, the #InfSession::synchronization-complete signal
  * is emitted.
  *
  * After subscription, one can observe modifications other users make, but it is
  * not possible to make own modifications. Before doing so, a #InfUser needs to
  * be joined. This is done by client/server specific API such as
- * infc_session_proxy_join_user() or infd_session_proxy_join_user(). The
+ * infc_session_proxy_join_user() or infd_session_proxy_add_user(). The
  * required parameters still depend on the actual note type, which is why most
  * note implementations offer their own API to join a user.
  **/
@@ -318,23 +318,13 @@ inf_session_send_sync_error(InfSession* session,
 {
   InfSessionPrivate* priv;
   xmlNodePtr node;
-  gchar code_buf[16];
 
   priv = INF_SESSION_PRIVATE(session);
 
   g_return_if_fail(priv->status == INF_SESSION_SYNCHRONIZING);
   g_return_if_fail(priv->shared.sync.conn != NULL);
 
-  node = xmlNewNode(NULL, (const xmlChar*)"sync-error");
-
-  xmlNewProp(
-     node,
-    (const xmlChar*)"domain",
-    (const xmlChar*)g_quark_to_string(error->domain)
-  );
-
-  sprintf(code_buf, "%u", (unsigned int)error->code);
-  xmlNewProp(node, (const xmlChar*)"code", (const xmlChar*)code_buf);
+  node = inf_xml_util_new_node_from_error(error, NULL, "sync-error");
 
   inf_communication_group_send_message(
     priv->shared.sync.group,
@@ -1469,11 +1459,8 @@ inf_session_communication_object_received(InfCommunicationObject* comm_object,
   InfSessionPrivate* priv;
   InfSessionSync* sync;
   gboolean result;
-  GQuark domain;
-  InfSessionSyncError code;
-  xmlChar* domain_attr;
-  xmlChar* code_attr;
   GError* local_error;
+  const gchar* local_message;
 
   session = INF_SESSION(comm_object);
   priv = INF_SESSION_PRIVATE(session);
@@ -1532,27 +1519,23 @@ inf_session_communication_object_received(InfCommunicationObject* comm_object,
     {
       if(strcmp((const gchar*)node->name, "sync-error") == 0)
       {
-        local_error = NULL;
-
         /* There was an error during synchronization, cancel remaining
          * messages. */
         inf_communication_group_cancel_messages(sync->group, connection);
 
-        domain_attr = xmlGetProp(node, (const xmlChar*)"domain");
-        code_attr = xmlGetProp(node, (const xmlChar*)"code");
+        local_error = inf_xml_util_new_error_from_node(node);
 
-        if(domain_attr != NULL && code_attr != NULL)
+        if(local_error != NULL)
         {
-          domain = g_quark_from_string((const gchar*)domain_attr);
-          code = strtoul((const gchar*)code_attr, NULL, 0);
-
-          g_set_error(
-            &local_error,
-            domain,
-            code,
-            "%s",
-            inf_session_get_sync_error_message(domain, code)
-          );
+          local_message =
+            inf_session_get_sync_error_message(local_error->domain,
+                                               local_error->code);
+          if(local_message != NULL)
+          {
+            if(local_error->message != NULL)
+              g_free(local_error->message);
+            local_error->message = g_strdup(local_message);
+          }
         }
         else
         {
@@ -1565,9 +1548,6 @@ inf_session_communication_object_received(InfCommunicationObject* comm_object,
           );
         }
 
-        if(domain_attr != NULL) xmlFree(domain_attr);
-        if(code_attr != NULL) xmlFree(code_attr);
-
         /* Note the default handler actually removes the sync */
         g_signal_emit(
           G_OBJECT(session),
@@ -2104,9 +2084,9 @@ inf_session_class_init(gpointer g_class,
    * This signal is emitted whenever a new XML node has been sent or received
    * over @connection as part of a synchronization. The process is completed
    * when @progress reaches the value 1.0. At this point,
-   * #InfSession::synchronisation-complete is also emitted.
+   * #InfSession::synchronization-complete is also emitted.
    *
-   * If @sessions&apos;s status is %INF_SESSION_SYNCHRONIZING, the local
+   * If @session&apos;s status is %INF_SESSION_SYNCHRONIZING, the local
    * side is being synchronized by the remote side. If the status is
    * %INF_SESSION_RUNNING, the local side is updating the remote side.
    */
@@ -2535,7 +2515,7 @@ inf_session_add_user(InfSession* session,
  * Changes the status of the given @user which needs to have the
  * %INF_USER_LOCAL flag set for this function to be called. If the status
  * is changed to %INF_USER_UNAVAILABLE, then the user leaves the session. To
- * rejoin use infc_session_proxy_join_user or infd_session_proxy_join_user,
+ * rejoin use infc_session_proxy_join_user() or infd_session_proxy_add_user(),
  * respectively for a proxy proxying @session.
  **/
 void
