@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,9 +23,11 @@
 #include <libinfinity/common/inf-tcp-connection.h>
 #include <libinfinity/common/inf-certificate-chain.h>
 #include <libinfinity/common/inf-certificate-credentials.h>
+#include <libinfinity/common/inf-sasl-context.h>
 
+#include <unistd.h> /* Get ssize_t on MSVC, required by gnutls.h */
 #include <gnutls/gnutls.h>
-#include <gsasl.h>
+/*#include <gsasl.h>*/
 
 #include <glib-object.h>
 
@@ -155,7 +157,7 @@ inf_xmpp_connection_new(InfTcpConnection* tcp,
                         const gchar* remote_hostname,
                         InfXmppConnectionSecurityPolicy security_policy,
                         InfCertificateCredentials* creds,
-                        Gsasl* sasl_context,
+                        InfSaslContext* sasl_context,
                         const gchar* sasl_mechanisms);
 
 gboolean
@@ -172,6 +174,22 @@ inf_xmpp_connection_certificate_verify_continue(InfXmppConnection* xmpp);
 void
 inf_xmpp_connection_certificate_verify_cancel(InfXmppConnection* xmpp);
 
+void
+inf_xmpp_connection_reset_sasl_authentication(InfXmppConnection* xmpp,
+                                              InfSaslContext* new_context,
+                                              const gchar* new_mechanisms);
+
+gboolean
+inf_xmpp_connection_retry_sasl_authentication(InfXmppConnection* xmpp,
+                                              GError** error);
+
+void
+inf_xmpp_connection_set_sasl_error(InfXmppConnection* xmpp,
+                                   const GError* error);
+
+const GError*
+inf_xmpp_connection_get_sasl_error(InfXmppConnection* xmpp);
+
 G_END_DECLS
 
 #endif /* __INF_XMPP_CONNECTION_H__ */
