@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -207,24 +207,69 @@ inf_communication_central_method_send_all(InfCommunicationMethod* method,
                                           xmlNodePtr xml)
 {
   InfCommunicationCentralMethodPrivate* priv;
+  InfCommunicationRegistry* registry;
+  InfCommunicationGroup* group;
+  GSList* connections;
   GSList* item;
-  GSList* next;
+  InfXmlConnection* connection;
+  gboolean is_registered;
 
   priv = INF_COMMUNICATION_CENTRAL_METHOD_PRIVATE(method);
 
-  for(item = priv->connections; item != NULL; item = next)
-  {
-    next = item->next;
+  /* Each of the inf_communication_registry_send() calls can do a callback
+   * which might possibly screw up our connection list completely. So be safe
+   * here by copying all relevant information on the stack. */
+  g_object_ref(method);
+  registry = g_object_ref(priv->registry);
+  group = g_object_ref(priv->group);
 
-    inf_communication_registry_send(
-      priv->registry,
-      priv->group,
-      INF_XML_CONNECTION(item->data),
-      next ? xmlCopyNode(xml, 1) : xml
+  connections = g_slist_copy(priv->connections);
+  for(item = connections; item != NULL; item = item->next)
+    g_object_ref(item->data);
+
+  while(connections)
+  {
+    connection = INF_XML_CONNECTION(connections->data);
+
+    /* A callback from a prior iteration might have unregistered the
+     * connection. */
+    is_registered = inf_communication_registry_is_registered(
+      registry,
+      group,
+      connection
     );
+
+    if(is_registered)
+    {
+      if(connections->next != NULL)
+      {
+        /* Keep ownership of XML if there might be more connections we should
+         * send it to. */
+        inf_communication_registry_send(
+          registry,
+          group,
+          connection,
+          xmlCopyNode(xml, 1)
+        );
+      }
+      else
+      {
+        /* Pass ownership of XML if this is definitely the last connection
+         * in the list. */
+        inf_communication_registry_send(registry, group, connection, xml);
+        xml = NULL;
+      }
+    }
+
+    g_object_unref(connection);
+    connections = g_slist_delete_link(connections, connections);
   }
 
-  if(priv->connections == NULL)
+  g_object_unref(method);
+  g_object_unref(registry);
+  g_object_unref(group);
+
+  if(xml != NULL)
     xmlFreeNode(xml);
 }
 
@@ -242,7 +287,7 @@ inf_communication_central_method_cancel_messages(InfCommunicationMethod* meth,
   );
 }
 
-static void
+static InfCommunicationScope
 inf_communication_central_method_received(InfCommunicationMethod* method,
                                           InfXmlConnection* connection,
                                           xmlNodePtr xml)
@@ -271,7 +316,7 @@ inf_communication_central_method_received(InfCommunicationMethod* method,
     );
 
     /* TODO: Find a better place to show this message, maybe in
-     * InfCommunicationGroup, somehow. */
+     * InfCommunicationRegistry's received_cb. */
     if(error != NULL)
     {
       buffer = xmlBufferCreate();
@@ -314,9 +359,15 @@ inf_communication_central_method_received(InfCommunicationMethod* method,
           );
         }
       }
-
-      /* TODO: Forward to other methods of same group */
     }
+
+    return scope;
+  }
+  else
+  {
+    /* Without target we don't know what to do with the message, so better
+     * don't relay. */
+    return INF_COMMUNICATION_SCOPE_PTP;
   }
 }
 
