@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,6 +21,10 @@
 #define __INF_COMMUNICATION_GROUP_PRIVATE_H__
 
 #include <libinfinity/communication/inf-communication-group.h>
+#include <libinfinity/communication/inf-communication-method.h>
+
+typedef void(*InfCommunicationGroupForeachFunc)(InfCommunicationMethod* meth,
+                                                gpointer user_data);
 
 void
 _inf_communication_group_add_member(InfCommunicationGroup* group,
@@ -30,6 +34,11 @@ void
 _inf_communication_group_remove_member(InfCommunicationGroup* group,
                                        InfXmlConnection* connection);
 
+void
+_inf_communication_group_foreach_method(InfCommunicationGroup* group,
+                                        InfCommunicationGroupForeachFunc func,
+					gpointer user_data);
+
 #endif /* __INF_COMMUNICATION_GROUP_PRIVATE_H__ */
 
 /* vim:set et sw=2 ts=2: */
