@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -70,6 +70,9 @@ struct _InfdDirectory {
   GObject parent;
 };
 
+typedef void(*InfdDirectoryForeachConnectionFunc)(InfXmlConnection*,
+                                                  gpointer);
+
 GType
 infd_directory_iter_get_type(void) G_GNUC_CONST;
 
@@ -109,6 +112,11 @@ gboolean
 infd_directory_add_connection(InfdDirectory* directory,
                               InfXmlConnection* connection);
 
+void
+infd_directory_foreach_connection(InfdDirectory* directory,
+                                  InfdDirectoryForeachConnectionFunc func,
+                                  gpointer user_data);
+
 const gchar*
 infd_directory_iter_get_name(InfdDirectory* directory,
                              InfdDirectoryIter* iter);
