@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -1194,14 +1194,9 @@ infd_session_proxy_communication_object_received(InfCommunicationObject* obj,
       seq = NULL;
 
     /* Only send request-failed when it was a proxy-related request */
-    reply_xml = xmlNewNode(NULL, (const xmlChar*)"request-failed");
-    inf_xml_util_set_attribute_uint(reply_xml, "code", local_error->code);
-
-    xmlNewProp(
-      reply_xml,
-      (const xmlChar*)"domain",
-      (const xmlChar*)g_quark_to_string(local_error->domain)
-    );
+    reply_xml = inf_xml_util_new_node_from_error(local_error,
+                                                 NULL,
+                                                 "request-failed");
 
     if(seq != NULL) inf_xml_util_set_attribute(reply_xml, "seq", seq);
     g_free(seq);
@@ -1447,7 +1442,7 @@ infd_session_proxy_add_user(InfdSessionProxy* proxy,
 /**
  * infd_session_proxy_subscribe_to:
  * @proxy: A #InfdSessionProxy.
- * @connection: A #InfConnection that is not yet subscribed.
+ * @connection: A #InfXmlConnection that is not yet subscribed.
  * @seq_id: The sequence identifier for @connection.
  * @synchronize: If %TRUE, then synchronize the session to @connection first.
  *
