@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,8 +24,8 @@
 #include <libinfinity/common/inf-xmpp-connection.h>
 #include <libinfinity/common/inf-certificate-credentials.h>
 
+#include <unistd.h> /* Get ssize_t on MSVC, required by gnutls.h */
 #include <gnutls/gnutls.h>
-#include <gsasl.h>
 
 #include <glib-object.h>
 
@@ -60,7 +60,7 @@ InfdXmppServer*
 infd_xmpp_server_new(InfdTcpServer* tcp,
                      InfXmppConnectionSecurityPolicy policy,
                      InfCertificateCredentials* creds,
-                     Gsasl* sasl_context,
+                     InfSaslContext* sasl_context,
                      const gchar* sasl_mechanisms);
 
 void
