@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -50,6 +50,16 @@ struct _InfTextBufferIface {
                             guint pos,
                             guint len);
 
+  void(*insert_text)(InfTextBuffer* buffer,
+                     guint pos,
+                     InfTextChunk* chunk,
+                     InfUser* user);
+
+  void(*erase_text)(InfTextBuffer* buffer,
+                    guint pos,
+                    guint len,
+                    InfUser* user);
+
   InfTextBufferIter*(*create_iter)(InfTextBuffer* buffer);
 
   void(*destroy_iter)(InfTextBuffer* buffer,
@@ -74,15 +84,15 @@ struct _InfTextBufferIface {
                           InfTextBufferIter* iter);
 
   /* Signals */
-  void(*insert_text)(InfTextBuffer* buffer,
+  void(*text_inserted)(InfTextBuffer* buffer,
+                       guint pos,
+                       InfTextChunk* chunk,
+                       InfUser* user);
+
+  void(*text_erased)(InfTextBuffer* buffer,
                      guint pos,
                      InfTextChunk* chunk,
                      InfUser* user);
-
-  void(*erase_text)(InfTextBuffer* buffer,
-                    guint pos,
-                    guint len,
-                    InfUser* user);
 };
 
 GType
@@ -150,6 +160,18 @@ guint
 inf_text_buffer_iter_get_author(InfTextBuffer* buffer,
                                 InfTextBufferIter* iter);
 
+void
+inf_text_buffer_text_inserted(InfTextBuffer* buffer,
+                              guint pos,
+                              InfTextChunk* chunk,
+                              InfUser* user);
+
+void
+inf_text_buffer_text_erased(InfTextBuffer* buffer,
+                            guint pos,
+                            InfTextChunk* chunk,
+                            InfUser* user);
+
 G_END_DECLS
 
 #endif /* __INF_TEXT_BUFFER_H__ */
