@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -122,7 +122,8 @@ inf_text_user_get_property(GObject* object,
 static void
 inf_text_user_selection_changed(InfTextUser* user,
                                 guint position,
-                                guint length)
+                                guint length,
+                                gboolean by_request)
 {
   InfTextUserPrivate* priv;
   priv = INF_TEXT_USER_PRIVATE(user);
@@ -201,11 +202,12 @@ inf_text_user_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfTextUserClass, selection_changed),
     NULL, NULL,
-    inf_marshal_VOID__UINT_INT,
+    inf_marshal_VOID__UINT_INT_BOOLEAN,
     G_TYPE_NONE,
-    2,
+    3,
     G_TYPE_UINT,
-    G_TYPE_INT
+    G_TYPE_INT,
+    G_TYPE_BOOLEAN
   );
 }
 
@@ -240,6 +242,40 @@ inf_text_user_get_type(void)
   return user_type;
 }
 
+/**
+ * inf_text_user_new:
+ * @id: The user ID for this user.
+ * @name: The user's name.
+ * @vector: The state at which the user is at, or %NULL.
+ * @hue: The hue value of the user's color.
+ *
+ * Creates a new #InfTextUser. @id should be unique for all users working
+ * together. #InfUserTable will refuse to add users with duplicate id. If
+ * @vector is %NULL, then the vector with all components zero is used.
+ *
+ * Returns: A new #InfTextUser. Free with g_object_unref() when no longer
+ * needed.
+ */
+InfTextUser*
+inf_text_user_new(guint id,
+                  const gchar* name,
+                  InfAdoptedStateVector* vector,
+                  double hue)
+{
+  g_return_val_if_fail(name != NULL, NULL);
+
+  return INF_TEXT_USER(
+    g_object_new(
+      INF_TEXT_TYPE_USER,
+      "id", id,
+      "name", name,
+      "vector", vector,
+      "hue", hue,
+      NULL
+    )
+  );
+}
+
 /**
  * inf_text_user_get_caret_position:
  * @user: A #InfTextUser.
@@ -284,7 +320,8 @@ inf_text_user_get_selection_length(InfTextUser* user)
 void
 inf_text_user_set_selection(InfTextUser* user,
                             guint position,
-                            gint length)
+                            gint length,
+                            gboolean by_request)
 {
   g_return_if_fail(INF_TEXT_IS_USER(user));
   g_signal_emit(
@@ -292,7 +329,8 @@ inf_text_user_set_selection(InfTextUser* user,
     user_signals[SELECTION_CHANGED],
     0,
     position,
-    length
+    length,
+    by_request
   );
 }
 
