@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -41,7 +41,8 @@ struct _InfTextUserClass {
 
   void(*selection_changed)(InfTextUser* user,
                            guint position,
-                           guint length);
+                           guint length,
+                           gboolean by_request);
 };
 
 struct _InfTextUser {
@@ -51,6 +52,12 @@ struct _InfTextUser {
 GType
 inf_text_user_get_type(void) G_GNUC_CONST;
 
+InfTextUser*
+inf_text_user_new(guint id,
+                  const gchar* name,
+                  InfAdoptedStateVector* vector,
+                  double hue);
+
 guint
 inf_text_user_get_caret_position(InfTextUser* user);
 
@@ -60,7 +67,8 @@ inf_text_user_get_selection_length(InfTextUser* user);
 void
 inf_text_user_set_selection(InfTextUser* user,
                             guint position,
-                            gint length);
+                            gint length,
+                            gboolean by_request);
 
 gdouble
 inf_text_user_get_hue(InfTextUser* user);
