@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,8 +24,7 @@
 #include <libinfinity/common/inf-user-table.h>
 #include <libinfinity/common/inf-user.h>
 
-#include <gtk/gtktextbuffer.h>
-
+#include <gtk/gtk.h>
 #include <glib-object.h>
 
 G_BEGIN_DECLS
@@ -69,6 +68,10 @@ InfTextUser*
 inf_text_gtk_buffer_get_author(InfTextGtkBuffer* buffer,
                                GtkTextIter* location);
 
+InfTextUser*
+inf_text_gtk_buffer_get_user_for_tag(InfTextGtkBuffer* buffer,
+                                     GtkTextTag* tag);
+
 gboolean
 inf_text_gtk_buffer_is_author_toggle(InfTextGtkBuffer* buffer,
                                      const GtkTextIter* iter,
