@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,6 +18,8 @@
  */
 
 #include <libinftextgtk/inf-text-gtk-buffer.h>
+#include <libinftextgtk/inf-text-gtk-view.h>
+#include <libinftextgtk/inf-text-gtk-viewport.h>
 #include <libinfgtk/inf-gtk-browser-view.h>
 #include <libinfgtk/inf-gtk-browser-store.h>
 #include <libinfgtk/inf-gtk-chat.h>
@@ -28,7 +30,9 @@
 #include <libinfinity/common/inf-discovery-avahi.h>
 #include <libinfinity/common/inf-chat-session.h>
 #include <libinfinity/common/inf-error.h>
+#include <libinfinity/common/inf-protocol.h>
 #include <libinfinity/inf-signals.h>
+#include <libinfinity/common/inf-init.h>
 
 #include <gtk/gtk.h>
 
@@ -39,8 +43,11 @@ struct _InfTestGtkBrowserWindow {
   GtkWidget* redo_button;
 
   InfTextGtkBuffer* buffer;
+  InfTextGtkView* view;
+  InfTextGtkViewport* viewport;
   InfcSessionProxy* proxy;
   InfUser* user;
+  InfcUserRequest* request;
 };
 
 typedef struct _InfTestGtkBrowserChatWindow InfTestGtkBrowserChatWindow;
@@ -51,6 +58,7 @@ struct _InfTestGtkBrowserChatWindow {
   InfChatBuffer* buffer;
   InfcSessionProxy* proxy;
   InfUser* user;
+  InfcUserRequest* request;
 };
 
 static InfSession*
@@ -176,6 +184,10 @@ on_chat_join_finished(InfcUserRequest* request,
   test->user = user;
   g_object_ref(user);
 
+  g_assert(test->request != NULL);
+  g_object_unref(test->request);
+  test->request = NULL;
+
   /* Unfortunately, gtk_widget_grab_focus(test->chat) +
    * gtk_container_set_focus_child() in inf_gtk_chat_set_active_user() does
    * not do the job which is why I added this crappy API. */
@@ -196,11 +208,17 @@ on_join_finished(InfcUserRequest* request,
   test = (InfTestGtkBrowserWindow*)user_data;
 
   inf_text_gtk_buffer_set_active_user(test->buffer, INF_TEXT_USER(user));
+  inf_text_gtk_view_set_active_user(test->view, INF_TEXT_USER(user));
+  inf_text_gtk_viewport_set_active_user(test->viewport, INF_TEXT_USER(user));
   gtk_text_view_set_editable(GTK_TEXT_VIEW(test->textview), TRUE);
 
   test->user = user;
   g_object_ref(user);
 
+  g_assert(test->request != NULL);
+  g_object_unref(test->request);
+  test->request = NULL;
+
   session = INF_ADOPTED_SESSION(infc_session_proxy_get_session(test->proxy));
   algorithm = inf_adopted_session_get_algorithm(session);
 
@@ -226,10 +244,14 @@ on_chat_join_failed(InfcRequest* request,
 
   test = (InfTestGtkBrowserChatWindow*)user_data;
 
+  g_assert(test->request != NULL);
+  g_object_unref(test->request);
+  test->request = NULL;
+
   if(error->domain == inf_user_error_quark() &&
      error->code == INF_USER_ERROR_NAME_IN_USE)
   {
-    new_name = g_strdup_printf("%s%d", g_get_user_name(), 2);
+    new_name = g_strdup_printf("%s%d", g_get_user_name(), rand());
     request_chat_join(test, new_name);
     g_free(new_name);
   }
@@ -255,10 +277,14 @@ on_join_failed(InfcRequest* request,
 
   test = (InfTestGtkBrowserWindow*)user_data;
 
+  g_assert(test->request != NULL);
+  g_object_unref(test->request);
+  test->request = NULL;
+
   if(error->domain == inf_user_error_quark() &&
      error->code == INF_USER_ERROR_NAME_IN_USE)
   {
-    new_name = g_strdup_printf("%s%d", g_get_user_name(), 2);
+    new_name = g_strdup_printf("%s%d", g_get_user_name(), rand());
     request_join(test, new_name);
     g_free(new_name);
   }
@@ -310,6 +336,10 @@ request_chat_join(InfTestGtkBrowserChatWindow* test,
       G_CALLBACK(on_chat_join_finished),
       test
     );
+
+    g_assert(test->request == NULL);
+    test->request = request;
+    g_object_ref(test->request);
   }
 }
 
@@ -375,6 +405,10 @@ request_join(InfTestGtkBrowserWindow* test,
       G_CALLBACK(on_join_finished),
       test
     );
+
+    g_assert(test->request == NULL);
+    test->request = request;
+    g_object_ref(request);
   }
 }
 
@@ -467,6 +501,23 @@ on_chat_window_destroy(GtkWindow* window,
     test
   );
 
+  if(test->request != NULL)
+  {
+    inf_signal_handlers_disconnect_by_func(
+      test->request,
+      G_CALLBACK(on_chat_join_finished),
+      test
+    );
+
+    inf_signal_handlers_disconnect_by_func(
+      test->request,
+      G_CALLBACK(on_chat_join_failed),
+      test
+    );
+
+    g_object_unref(test->request);
+  }
+
   if(test->proxy != NULL) g_object_unref(test->proxy);
   if(test->user != NULL) g_object_unref(test->user);
 
@@ -495,6 +546,23 @@ on_text_window_destroy(GtkWindow* window,
     test
   );
 
+  if(test->request != NULL)
+  {
+    inf_signal_handlers_disconnect_by_func(
+      test->request,
+      G_CALLBACK(on_join_finished),
+      test
+    );
+
+    inf_signal_handlers_disconnect_by_func(
+      test->request,
+      G_CALLBACK(on_join_failed),
+      test
+    );
+
+    g_object_unref(test->request);
+  }
+
   if(test->proxy != NULL) g_object_unref(test->proxy);
   /* TODO: Do we ever ref buffer? */
   if(test->buffer != NULL) g_object_unref(test->buffer);
@@ -546,6 +614,7 @@ on_subscribe_chat_session(InfcBrowser* browser,
   test->buffer = buffer;
   test->proxy = proxy;
   test->user = NULL;
+  test->request = NULL;
   g_object_ref(test->proxy);
 
   g_signal_connect_after(
@@ -584,7 +653,11 @@ on_subscribe_session(InfcBrowser* browser,
   GtkWidget* undo_button;
   GtkWidget* redo_button;
   InfSession* session;
+  InfIo* io;
+  InfUserTable* user_table;
   InfTextGtkBuffer* buffer;
+  InfTextGtkView* view;
+  InfTextGtkViewport* viewport;
   GtkTextBuffer* textbuffer;
   InfTestGtkBrowserWindow* test;
 
@@ -595,14 +668,18 @@ on_subscribe_session(InfcBrowser* browser,
   }
 
   session = infc_session_proxy_get_session(proxy);
+  io = inf_adopted_session_get_io(INF_ADOPTED_SESSION(session));
+  user_table = inf_session_get_user_table(session);
   buffer = INF_TEXT_GTK_BUFFER(inf_session_get_buffer(session));
   textbuffer = inf_text_gtk_buffer_get_text_buffer(buffer);
 
   textview = gtk_text_view_new_with_buffer(textbuffer);
+  view = inf_text_gtk_view_new(io, GTK_TEXT_VIEW(textview), user_table);
   gtk_text_view_set_editable(GTK_TEXT_VIEW(textview), FALSE);
   gtk_widget_show(textview);
 
   scroll = gtk_scrolled_window_new(NULL, NULL);
+  viewport = inf_text_gtk_viewport_new(GTK_SCROLLED_WINDOW(scroll), user_table);
   gtk_scrolled_window_set_shadow_type(
     GTK_SCROLLED_WINDOW(scroll),
     GTK_SHADOW_IN
@@ -652,8 +729,11 @@ on_subscribe_session(InfcBrowser* browser,
   test->undo_button = undo_button;
   test->redo_button = redo_button;
   test->buffer = buffer;
+  test->view = view;
+  test->viewport = viewport;
   test->proxy = proxy;
   test->user = NULL;
+  test->request = NULL;
   g_object_ref(proxy);
 
   g_signal_connect_after(
@@ -701,7 +781,7 @@ on_activate(InfGtkBrowserView* view,
   InfcBrowserIter* browser_iter;
 
   gtk_tree_model_get(
-    GTK_TREE_MODEL(inf_gtk_browser_view_get_model(view)),
+    GTK_TREE_MODEL(gtk_tree_view_get_model(GTK_TREE_VIEW(view))),
     iter,
     INF_GTK_BROWSER_MODEL_COL_BROWSER, &browser,
     INF_GTK_BROWSER_MODEL_COL_NODE, &browser_iter,
@@ -788,9 +868,21 @@ main(int argc,
   GtkWidget* view;
   GtkWidget* scroll;
   GtkWidget* window;
+  GError* error;
+
+  int i;
+  InfTcpConnection* tcp;
+  InfXmppConnection* xmpp;
+  InfIpAddress* addr;
 
   gtk_init(&argc, &argv);
-  gnutls_global_init();
+
+  error = NULL;
+  if(!inf_init(&error))
+  {
+    fprintf(stderr, "%s\n", error->message);
+    return -1;
+  }
 
   io = inf_gtk_io_new();
 #ifdef LIBINFINITY_HAVE_AVAHI
@@ -802,7 +894,6 @@ main(int argc,
   communication_manager = inf_communication_manager_new();
   store = inf_gtk_browser_store_new(INF_IO(io), communication_manager);
   g_object_unref(communication_manager);
-  g_object_unref(G_OBJECT(io));
 
   g_signal_connect_after(
     G_OBJECT(store),
@@ -811,6 +902,43 @@ main(int argc,
     NULL
   );
 
+  for(i = 1; i < argc; ++i)
+  {
+    addr = inf_ip_address_new_from_string(argv[i]);
+
+    tcp = inf_tcp_connection_new(
+      INF_IO(io),
+      addr,
+      inf_protocol_get_default_port()
+    );
+
+    xmpp = inf_xmpp_connection_new(
+      tcp,
+      INF_XMPP_CONNECTION_CLIENT,
+      g_get_host_name(),
+      argv[i],
+      INF_XMPP_CONNECTION_SECURITY_BOTH_PREFER_TLS,
+      NULL,
+      NULL,
+      NULL
+    );
+
+    inf_ip_address_free(addr);
+    g_object_unref(tcp);
+
+#ifdef LIBINFINITY_HAVE_AVAHI
+    inf_xmpp_manager_add_connection(xmpp_manager, xmpp);
+#endif
+
+    inf_gtk_browser_store_add_connection(
+      INF_GTK_BROWSER_STORE(store),
+      INF_XML_CONNECTION(xmpp),
+      argv[i]
+    );
+  }
+
+  g_object_unref(G_OBJECT(io));
+
 #ifdef LIBINFINITY_HAVE_AVAHI
   inf_gtk_browser_store_add_discovery(store, INF_DISCOVERY(avahi));
   g_object_unref(G_OBJECT(avahi));
